@@ -3,10 +3,7 @@ from .schema import *
 from .api import *
 from ._deprecated import *
 
-from ...datasets import (
-    list_datasets,
-    load_dataset
-)
+from ...datasets import list_datasets, load_dataset
 
 from ... import expr
 from ...expr import datum
@@ -15,8 +12,13 @@ from .display import VegaLite, renderers
 
 from .data import (
     MaxRowsError,
-    pipe, curry, limit_rows,
-    sample, to_json, to_csv, to_values,
+    pipe,
+    curry,
+    limit_rows,
+    sample,
+    to_json,
+    to_csv,
+    to_values,
     default_data_transformer,
-    data_transformers
+    data_transformers,
 )
